#
#  Copyright 1996-2011 Jim Pirzyk / pirzyk.org
#  All rights reserved.
#
#  Security::OS::linux - Linux specific configuration information
#
package Security::OS::linux;

use Security::util::stat;
use Security::util::users;
use Security::util::groups;
use Security::util::uniq;
use Exporter;
use strict;
use warnings;
use Fcntl qw(:mode);

our @ISA = qw(Exporter);
our @EXPORT_OK = qw( &init() &check );

our (%perms, %files, %pkgs);
our $rpm = '/bin/rpm';
our $verifyargs = '--verify --all --nomd5 --nomtime --nosignature --nodeps --nodigest --noscripts --nosize';

sub init {
	print ( STDERR  "In Security::OS::linux::init()\n") if ( $main::opts{'d'} );

	&main::read_config('ignorepkgfiles');
	&main::read_config('nopkgduperr');

	$main::showmount = '/usr/sbin/showmount';
	$main::exportsfile = '/etc/exports';
	$main::securefile='/etc/securetty';
	$main::kerneldir='/lib/modules';
	$main::sudofile = '/etc/sudoers';
}

sub check {
	print ( STDERR  "In Security::OS::linux::check()\n") if ( $main::opts{'d'} );

	&Security::rootlogin::check_securettys() 
		if ( defined &Security::rootlogin::check_securettys );

	&check_osfiles() if ( -f $rpm );
}

sub check_osfiles {
	print ( STDERR  "In Security::OS::linux::check_osfiles()\n") if ( $main::opts{'d'} );

	_get_errors();
	_find_pkg_info();

	# Now go through and tell us what is wrong....
	_save_errors();
}

sub print {
	my ($str);
	print ( STDERR  "In Security::OS::linux::print()\n") if ( $main::opts{'d'} );

	# Flatten out the perl stucture for printing...
	for my $file ( keys %perms ) {
		if ( scalar keys %{$perms{$file}} ) {
			for my $type ( keys %{$perms{$file}} ) {
				if ( exists $perms{$file}->{$type}->{'is'} ) { 
					$str = _format_line($type, $perms{$file}->{$type}->{'is'}, &Security::util::uniq::uniq(@{$perms{$file}->{$type}->{'should'}}));
					delete $perms{$file}->{$type}->{'is'};
				} else {
					$str = _format_line($type, 'N/A', &Security::util::uniq::uniq(@{$perms{$file}->{$type}->{'should'}}));
				}
				delete $perms{$file}->{$type}->{'should'};
				$perms{$file}->{$type} = $str;
			}
		} else {
			delete $perms{$file};
		}
	}

	&main::print_data ("\n\nchecking rpm check command output:\n", %perms);
}

sub _get_errors {
	open (FP, "$rpm $verifyargs|") || &main::security_die ("Could not run $rpm $verifyargs\n");
	while (<FP>) {
		chomp;
		s/ \(Permission denied\)$//;
		my ($fmode, $type, $file) = split;
		$file = $type if ( ! defined $file );

		if ( -e $file || $file =~ m|^/| ) { 
			if ( !grep (/^${file}$/, @main::ignorepkgfiles)) {
				if ( $fmode ne 'missing' ) {
					my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$curbytes) = &Security::util::stat::get_stat($file, 'lstat');
					my $owner = &Security::util::users::get('uid', $uid, 'name');
					my $group = &Security::util::groups::get('gid', $gid, 'name');
					$files{$file} = {
						size => $curbytes,
						mode => $mode,
						owner => (defined $owner? $owner: $uid),
						group => (defined $group? $group: $gid),
						rdev => $rdev,
						failure => $fmode,
					}
				} else {
					$files{$file}->{'failure'} = $fmode;
				}
			}
		} else {
			warn "Could not understand output ($_)\n";
		}
	}
	close FP;
}

sub _find_pkg_info {
	foreach my $file ( keys %files ) {
		foreach my $pkg (split (/\n/, `$rpm -qf --qf "%{NAME}\n" $file`)) {
			push @{$files{$file}->{pkg}}, $pkg;
			if ( ! exists $pkgs{$pkg} ) {
				open (FP, "$rpm -q --dump $pkg |") || &main::security_die ("Could not run $rpm -q --dump $pkg\n");
				while (<FP>) {
					chomp;
					my ($file, $size, $mtime, $md5, $mode, $owner, $group, $isconfig, $isdoc, $rdev, $symlink) = split;
					$pkgs{$pkg}->{$file} = {
						size => $size,
						mtime => $mtime,
						md5 => $md5,
						mode => oct($mode),
						owner => $owner,
						group => $group,
						rdev => $rdev,
						symlink => $symlink,
					}
				}
				close FP;
			}
		}
	}
}

sub _save_errors {
	foreach my $file ( keys %files ) {
		foreach my $pkg ( @{$files{$file}->{pkg}} ) {
			if ( exists $pkgs{$pkg} ) {
				if ( $files{$file}->{failure} eq 'missing' ) {
					push @{$perms{$file}->{'file type'}->{'should'}}, &_format_type($pkgs{$pkg}->{$file}->{'mode'});
					push @{$perms{$file}->{'mode'}->{'should'}}, &_format_mode($pkgs{$pkg}->{$file}->{'mode'});
					push @{$perms{$file}->{'device'}->{'should'}}, $pkgs{$pkg}->{$file}->{'rdev'}
						if ( $pkgs{$pkg}->{$file}->{'rdev'} > 0 );
					push @{$perms{$file}->{'link'}->{'should'}}, $pkgs{$pkg}->{$file}->{symlink}
						if (&S_ISLNK($pkgs{$pkg}->{$file}->{mode}));

					push @{$perms{$file}->{'group'}->{'should'}}, $pkgs{$pkg}->{$file}->{'group'};
					push @{$perms{$file}->{'owner'}->{'should'}}, $pkgs{$pkg}->{$file}->{'owner'};
				} else {
					if ( $files{$file}->{failure} =~ /M/ ) {
						if (&S_IFMT($pkgs{$pkg}->{$file}->{mode}) != &S_IFMT($files{$file}->{mode})) {
							$perms{$file}->{'file type'}->{'is'} = &_format_type($files{$file}->{'mode'});
							push @{$perms{$file}->{'file type'}->{'should'}}, &_format_type($pkgs{$pkg}->{$file}->{'mode'});
							if (  ${main::nopkgduperr[0]} eq 'yes' && $files{$file}->{'mode'} eq $pkgs{$pkg}->{$file}->{'mode'} ) {
								$files{$file}->{failure} =~ s/M/\./;
								delete $perms{$file}->{'file type'};
							}
						} else {
							$perms{$file}->{'mode'}->{'is'} = &_format_mode($files{$file}->{'mode'});
							push @{$perms{$file}->{'mode'}->{'should'}}, &_format_mode($pkgs{$pkg}->{$file}->{'mode'});
							if (  ${main::nopkgduperr[0]} eq 'yes' && $files{$file}->{'mode'} eq $pkgs{$pkg}->{$file}->{'mode'} ) {
								$files{$file}->{failure} =~ s/M/\./;
								delete $perms{$file}->{'mode'};
							}
						}
					}
					
					if ( $files{$file}->{failure} =~ /D/ ) {
						$perms{$file}->{'device'}->{'is'} = $files{$file}->{'rdev'};
						push @{$perms{$file}->{'device'}->{'should'}}, $pkgs{$pkg}->{$file}->{'rdev'};
						if (  ${main::nopkgduperr[0]} eq 'yes' && $files{$file}->{'rdev'} eq $pkgs{$pkg}->{$file}->{'rdev'} ) {
							$files{$file}->{failure} =~ s/D/\./;
							delete $perms{$file}->{'device'};
						}
					}

					if ( $files{$file}->{failure} =~ /L/ ) {
						if ( $pkgs{$pkg}->{$file}->{symlink} eq 'X' ) {
							push @{$perms{$file}->{'link'}->{'should'}}, 'should not be a link';
						} else {
							push @{$perms{$file}->{'link'}->{'should'}}, $pkgs{$pkg}->{$file}->{symlink};
						}
					}

					if ( $files{$file}->{failure} =~ /U/ ) {
						$perms{$file}->{'owner'}->{'is'} = $files{$file}->{'owner'};
						push @{$perms{$file}->{'owner'}->{'should'}}, $pkgs{$pkg}->{$file}->{'owner'};
						if (  ${main::nopkgduperr[0]} eq 'yes' && $files{$file}->{'owner'} eq $pkgs{$pkg}->{$file}->{'owner'} ) {
							$files{$file}->{failure} =~ s/U/\./;
							delete $perms{$file}->{'owner'};
						}
					}

					if ( $files{$file}->{failure} =~ /G/ ) {
						$perms{$file}->{'group'}->{'is'} = $files{$file}->{'group'};
						push @{$perms{$file}->{'group'}->{'should'}}, $pkgs{$pkg}->{$file}->{'group'};
						if (  ${main::nopkgduperr[0]} eq 'yes' && $files{$file}->{'group'} eq $pkgs{$pkg}->{$file}->{'group'} ) {
							$files{$file}->{failure} =~ s/G/\./;
							delete $perms{$file}->{'group'};
						}
					}
				}
			} else {
				&main::security_die ("Can't find package for file ($file)\n");
			}
		}
	}
}

sub _format_line {
	my ($type, $is, @should) = @_;
	my ($str);

	if ( $type eq 'link' ) {
		if ( grep /should not be a link/, @should ) {
			$str = 'should not be a link';
		} else {
			if ( scalar @should gt 1 ) {
				$str = ' -> {' . join ('|', sort(@should)) . '}';
			} else {
				$str = '-> ' . $should[0];
			}
		}
	} else {
		if ( scalar @should gt 1 ) {
			$str = '({' . join ('|', sort(@should)) . '}:' . $is . ')';
		} else {
			$str = '(' . $should[0] . ':' . $is . ')';
		}
	}

	return $str;
}

sub _format_type {
	my ($mode) = @_;
	return 'file' if (&S_ISREG($mode));
	return 'directory' if (&S_ISDIR($mode));
	return 'link' if (&S_ISLNK($mode));
	return 'socket' if (&S_ISSOCK($mode));
	return 'block' if (&S_ISBLK($mode));
	return 'character' if (&S_ISCHR($mode));
	return 'fifo' if (&S_ISFIFO($mode));
}

sub _format_mode {
	my ($mode) = @_;
	return sprintf "%4.4o", &S_IMODE($mode);
}

1;

__END__

=head1 NAME

Security::OS::linux

=head1 SYNOPSIS

Security::OS::linux::init();

Security::OS::linux::check();

=head1 DESCRIPTION

Sets variables for OS specific commands.

=head1 API

Defines the following API calls

=over 4

=item C<init()>

Init sets the following global variables:

=over 4

=item C<$main::showmount>

The location of the showmount command, used in the C<Security::exports> module.

=item C<$main::exportsfile>

The location of the exports file, used in C<Security::exports> module.

=item C<$main::sudofile>

The location of F<sudoers> file, used in C<Security::sudo> module.

=item C<$main::kerneldir>

The location of the kernel loadable modules, used in C<Security::kernel>
module.

=item C<$main::securefile>

The location of the list of ports root is allowed in on, used in
C<Security::rootlogin> module.

=back

=item C<check()>

Calls C<Security::rootlogin::check_securetty()> to check what root ports are
configured and compares it to what ports we know about.  If the C<rpm> command
is found on the system, it will C<rpm --verify...> to find inconsitancies
between the RPM database and what is on the filesystem.

=back

=head1 CONFIGURATION OPTIONS

The script will call C<read_config> on the following config file options:

=over 4

=item C<ignorepkgfiles>

List of files to ignore errors returned from C<rpm --verify>

=item C<nopkgduperr>

This option will skip package errors if the problem file has multiple
entries that do not match one of the entries.  For example if a directory
should be owned by C<root> in one package and by C<bin> in another and
the directory is owned by C<root>.

=back

=head1 SEE ALSO

L<Security::util::groups>

L<Security::util::stat>

L<Security::util::uniq>

L<Security::util::users>

L<rpm(8)>

=cut
