#
#  Copyright 1996-2011 Jim Pirzyk / pirzyk.org
#  All rights reserved.
#
#  Security::util::groups - Cached getgrent data
#
package Security::util::groups;

use User::grent;
use Exporter;
use strict;
use warnings;

our @ISA = qw(Exporter);
our @EXPORT_OK = qw( &init() &check() &get() );

our ($gids, $groups);

sub init {
	setgrent;
	while (my $gr = getgrent) {
		# Skip the YP map entry...
		if ( defined $gr && $gr->name ne '+' ) {

			# Apply patch for bug in perl's getgrent on AIX, wrt gid = -2
			$gr->gid = 4294967294
				if ($gr->gid == -2 &&  $^O =~ m/aix/);

			$groups->{$gr->name} = $gr;
			$gids->{$gr->gid} = $gr;
		}
	}
	endgrent;
}

sub _bygid {
	$groups->{$a}->gid <=> $groups->{$b}->gid;
}

sub get {
	my ($field, $value, $subfield) = @_;

	if ( defined $value && defined $subfield ) {
		if ( $field eq 'gid' && exists $gids->{$value} ) {
			return $gids->{$value}->$subfield;
		} elsif ( exists $groups->{$value} ) {
			return $groups->{$value}->$subfield;
		} else {
			return undef
		}
	} else {
		return $field eq 'gid' 
			? sort keys %{$gids}
			: sort _bygid keys %{$groups};
	}
}

sub check {
	my ($field, $value) = @_;
	my ($res);

	if ( $field eq 'gid' ) {
		$res = exists $gids->{$value};
	} else {
		$res = exists $groups->{$value};
	}

	return $res;
}

1;

__END__

=head1 NAME

Security::util::groups - module for checking against the group file.

=head1 SYNOPSIS

Security::util::groups::check($field, $value)

=head1 DESCRIPTION

A wrapper routine around C<User::grent::getgrent> calls to return the
group file information.

=head1 API

The API calls provided by this module

=over 4

=item C<init ()>

Reads in all the group information.

=item C<get ($field [, $value, $subfield])>

If I<$field> is the only argument, C<get()> will return a list all the
values, in gid order.  I<$field> can be either C<name> or C<gid>.  If
I<$value> and I<$subfield> are defined, C<get()> will treat $value as
either a login name or gid and return the specific $subfield value from
the group file.  

=item C<check ($field, $value)>

Returns true if I<$field> has I<$value>.  I<$field> can be either C<name>
or C<gid>.

=back

=head1 CONFIGURATION OPTIONS

None.

=head1 SEE ALSO

L<User::grent>

L<perl>

=cut
